package ch.inf3.horizoncut.data;

import java.util.Collection;

public class DisplayCalculator {

    final int width;
    final int height;

    final double minHorizAngle;
    final double maxHorizAngle;

    final Position eyePosition;

    double minVertAngle;
    double maxVertAngle;

    public DisplayCalculator(int width, int height, double minHorizAngle, double maxHorizAngle, Position eyePosition) {
        this.width = width;
        this.height = height;

        this.minHorizAngle = minHorizAngle;
        this.maxHorizAngle = maxHorizAngle;

        this.eyePosition = eyePosition;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public double getMinHorizAngle() {
        return minHorizAngle;
    }

    public double getMaxHorizAngle() {
        return maxHorizAngle;
    }

    public double getEyeHeight() {
        return eyePosition.getHeight();
    }

    public Position getEyePosition() {
        return eyePosition;
    }

    public double getAngleStep() {
        return (maxHorizAngle - minHorizAngle) / width;
    }

    public boolean isDisplayed(double bearing, double visibleAngle) {
        return (bearing <= maxHorizAngle && bearing >= minHorizAngle)
                && (visibleAngle <= maxVertAngle && visibleAngle >= minVertAngle);
    }

    /**
     *
     * @param x
     * @return Bearing in degrees
     */
    public double getBearingAtX(int x) {
        return (x * getAngleStep()) + minHorizAngle;
    }

    /**
     *
     * @param bearing Bearing in degrees
     * @return
     */
    public int getXAtBearing(double bearing) {
        return (int) ((bearing - minHorizAngle) / getAngleStep());
    }

    public int getYAtAngle(double visibleAngle) {
        double pos = (visibleAngle - minVertAngle) * (height / (maxVertAngle - minVertAngle));

        int y = (int) (height - pos - 1);

        if (y <= 0 || y > height) {
            y = height;
        }

        return y;
    }
    
    public void setMinMaxVertAngle(double minVertAngle, double maxVertAngle) {
        this.minVertAngle = minVertAngle;
        this.maxVertAngle = maxVertAngle;
    }

    public void updateMinMaxVertAngle(Collection<DistanceLayer> nLayers) {
        // We dont want to compte the min angle to avoid looking at the ground
        // just before the position ...
        this.minVertAngle = 80;
        this.maxVertAngle = nLayers.stream()
                .map(l -> l.getMax())
                .max(Double::compareTo)
                .orElse(0.0) + 3;
    }

    public double calculateVisibilityAngle(double targetVisibleHeight, int targetDistance) {
        if (eyePosition.getHeight() == targetVisibleHeight) {
            return 90.0;
        } else if (eyePosition.getHeight() < targetVisibleHeight) {
            double op = targetVisibleHeight - eyePosition.getHeight();
            double ad = targetDistance;
            return Math.toDegrees(Math.atan(op / ad)) + 90;
        } else {
            return Math.toDegrees(Math.atan(targetDistance / (eyePosition.getHeight() - targetVisibleHeight)));
        }
    }
}
