package ch.inf3.horizoncut.data;

public class ObservedFeature extends Feature implements Comparable<ObservedFeature> {

    private final int distance;
    private final double visibleAngle;
    private final double bearing;

    private final DisplayCalculator observator;

    public ObservedFeature(Position position, double elevation, DisplayCalculator observator, String name, int textPosition, boolean requireHorizon, double score) {
        super(position, elevation, name, textPosition, requireHorizon, score);

        this.observator = observator;

        this.distance = observator.eyePosition.distanceToMeters(position);
        this.visibleAngle = observator.calculateVisibilityAngle(elevation, distance);

        this.bearing = Math.toDegrees(Position.bearing(observator.eyePosition, position));
    }

    @Override
    public int compareTo(ObservedFeature other) {
        return Double.compare(this.bearing, other.bearing);
    }

    public int getObservedX() {
        return observator.getXAtBearing(bearing);
    }

    public int getObservedY() {
        return observator.getYAtAngle(visibleAngle);
    }
    
    public double getBearing() {
        return this.bearing;
    }
    
    public double getAngle() {
        return this.visibleAngle;
    }

    public int getDistance() {
        return distance;
    }
}
